/////////////////////////////////////////////////////////////////
/////
////	Optima++ sample for WStream class; tested with Optima++ 1.5
///
//	You may use this source code in any way you like;
//	it has been tested, but may still contain errors.
// 	In any case, its up to the user to make shure this
//	source works correctly for his/her purpose.
// 	In case you have questions concerning the file i/o used, 
//	look at the documentation	for the clib, or email me
//	at mlenzing@aol.com. If you find errors, please mail me
//	so I can correct them in my files and repost the updated files.
//	
//			** READ THE TEXT BELOW! **
//
// What is this:
//	its an example program, use the debugger to see what happens
//	at each instruction. The code is simple and easy to understand (I hope).
//  You might want to build a .lib file from the WStream sources
//	to include into your projects. Beware that Optima++ creates a 
// 	of these files if you include them directly into your targets.
//	This means that any change you make will only apply to the local copy.
//	I store my support files in a common directory and only
//	include a dummy file in the targets. The dummy consist of the
//	line:	
//		#include "\misc\wstream.cpp" (as an example)
//
//	Debugging still works fine with this.
//
// Notes on the format of this sources files:
//	If this file looks strangely formatted to you, it may be because of
//	the editor settings I use.
//	I use tabs, displayed at the size of two chars.
// 
// Note on WStream and its flags:
//	The only flags implemented in WStream are
//		WStream::stateLoading
//		WStream::stateSaving
//		WStream::stateOpen
//		WStream::stateError
//
//	if you want to use any other flags, you will have
//	to implement their meaning yourself.
//

/* [main.cpp|001]
 *  main.cpp (console applications)
*/

#include "wpch.hpp"
#include "wstream.hpp"


/*  Redefine the C++ "new" and "delete" operators to go through
    the component library's memory checker.
*/
#if defined( _DEBUG ) && defined( new )
#   undef new
#   undef delete
#   define _REDEFINE_WNEW
#endif
W_REDEFINENEW
W_REDEFINEDELETE
#ifdef _REDEFINE_WNEW
#   include "wnew.hpp"
#   undef _REDEFINE_WNEW
#endif


WModuleHandle _ApplicationModule = NULLHMODULE;
static WApplication _Application;
WApplication *Application = &_Application;

const	int COUNT = 16;

////
// this is an example to show how to 
// overload stream operators
//	<< and >> for objects.

class WSample : public WObject
{
	////
	// member vars...
protected:
	WChar* m_pszMsg;
	WBool m_bDoDelete;
	
public:
	// a simple ontructor
	WSample(WChar* pszMsg = "Some Message!!")	
		: WObject()
	{ 
		m_bDoDelete = FALSE;
		m_pszMsg = pszMsg; 
	};
	
	// and a simple destructor
	~WSample() 
	{
		Clear();
	};
	
	// clean up functions
	void Clear()
	{
		if (m_bDoDelete && m_pszMsg)
			delete[] m_pszMsg;
		m_pszMsg = NULL;
		m_bDoDelete = FALSE;
	};
	
	////
	// loading/storing via the stream shift in/out operators
	
	friend WStream& operator << (WStream& Stream, WSample& Sample);
	friend WStream& operator >> (WStream& Stream, WSample& Sample);
	
	////
	// function to implement the storing / loading...
	
protected:
	WSample& Load(WStream& Stream)
	{
		Clear();
		
		WULong cbLen = 0;
		Stream >> cbLen;
		
		if (cbLen)
		{
			WChar* psz = new WChar[cbLen+1];
			Stream.Read(psz, sizeof (WChar) * cbLen);
			psz[cbLen] = '\0';
			m_pszMsg = psz;
			m_bDoDelete = TRUE;
		} else m_pszMsg = "";
		
		return *this;
	};
	
	WSample& Save(WStream& Stream)
	{
		WULong cbLen = 0;
		
		if (!m_pszMsg)
			m_pszMsg = "";
			
		// use might want to use ::strlen...
		for(char* p = m_pszMsg; *p; p++)
			cbLen++;
			
		Stream << cbLen;
		Stream.Write(m_pszMsg, cbLen);
		
		return *this;
	};
	
	////
	// loading/storing via serialization function
	
	void Serialize(WStream& Stream)
	{
		if (Stream.IsLoading())
			Load(Stream);
		else
			Save(Stream);
	};
};

WStream& operator << (WStream& Stream, WSample& Sample)
	{ 
		Sample.Save(Stream);
		return Stream;
	};
	
WStream& operator >> (WStream& Stream, WSample& Sample)
	{
		Sample.Load(Stream);
		return Stream;
	};

void DoWStreamTest()
{
	WULong pos, len, center1, center2;
	
	// prepare some data...
	WULong data[COUNT];
	for (int i=0; i<COUNT; i++)
		data[i] = i;
	
	WFile* f = new WFile;
	WBool succ = f->Open("this.is-a-file-with.dots.in-its-name", WFile::modeCreate | WFile::modeWrite);
	
	if (succ)
	{


		
		// truncate the file to length 0...
		f->SetSize(0);
		
		// look at len to see if it worked...
		len = f->GetSize();
		
		// save data to file, hope the compiler gets it right...
		// 	data == &data[0];	sizeof(data) == sizeof (WULong) * COUNT;
		f->Write(data, sizeof(data) );
		
		// file will be closed if reopened...
		f->Open("this.is.another.file", WFile::modeCreate | WFile::modeReadWrite);
		f->SetSize(0);
		
		// also fill with data...
		for (i=0; i<COUNT; i++)
			*f << data[i];
			
		// seek to beginning...
		f->Seek(0);
		for (i=COUNT; i>0; )
			*f >> data[--i];
			
		// read the two values at the center...
		f->Seek(sizeof(WULong) * (COUNT / 2));
		*f >> center1;
		*f >> center2;
		
		f->Seek(- sizeof(WULong), WFile::current);
		*f >> center1;	// should be == center2 now...
		
		// read and write some WSample objects
		WSample Sample1("sample 1");
		WSample Sample2("sample 2");
		
		// append a WSample to end...
		f->Seek(0, WFile::end);
		pos = f->GetPos();
		
		// save...
		*f << Sample1;
		*f << Sample2;
		
		// load in reverse order...
		f->Seek(pos);
		*f >> Sample2;
		*f >> Sample1;

		// note that you have to use modeWrite or modeRead
		// (not modeReadWrite) to use WSample::Serialize

		////
		// this will also close the file.
		delete f;
	}
}

int main( int argc, const char *argv[] )
{
    int ret = -1;

    if( Application->Initialize( NULL, 0, NULL, 0 ) ) 
		{
			DoWStreamTest();

			Application->Uninitialize();
    }
    return ret;
}
