//////////////////////////////////////////////////////
/////
////	Optima++ class; tested with Optima++ 1.5
///
//	You may use this source code in any way you like;
//	it has been tested, but may still contain errors.
// 	In any case, its up to the user to make shure this
//	source works correctly for his/her purpose.
// 	In case you have questions concerning the file i/o used, 
//	look at the documentation	for the clib, or email me
//	at mlenzing@aol.com. If you find errors, please mail me
//	so I can correct them in my files and repost the updated files.
//	

#include "wstream.hpp"
#include <io.h>

///////////////////////////////////////
// stream base class

WStream::WStream()
 : WObject ()
{
	m_cbIn = m_cbOut = 0;
}

WStream::~WStream()
{
}

////
// stream access functions

WBool WStream::Open(WBool bSave)
{
	if (IsOpen())
		Close();
	
	if (bSave)
		SetFlag(stateSaving);
	else
		SetFlag(stateLoading);

	SetFlag(stateOpen);
		
	return TRUE;
}

WBool WStream::Close()
{
	ClearFlag(stateSaving);
	ClearFlag(stateLoading);
	ClearFlag(stateOpen);
	
	return TRUE;
}

int WStream::GetError()
{
	int nErr = m_nErr;
	m_nErr = 0;
	ClearFlag(stateError);
	return nErr;
}

void WStream::SetError(int nErr)
{
	if ( (m_nErr = nErr) != 0)
		SetFlag(stateError);
	else
		ClearFlag(stateError);
}

void WStream::Reset()
{
	SetError(0);
	m_cbIn = m_cbOut = 0;
}
	
////
// shift in/shift out operators:
	
void WStream::operator << (char& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (int& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (short& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (long& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (unsigned char&  v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (unsigned int& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (unsigned short& v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (unsigned long&  v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (float&  v)
{ Write(&v, sizeof(v)); };
void WStream::operator << (double& v)
{ Write(&v, sizeof(v)); };
  
void WStream::operator >> (char& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (int& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (short& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (long& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (unsigned char&  v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (unsigned int& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (unsigned short& v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (unsigned long&  v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (float&  v)
{ Read(&v, sizeof(v)); };
void WStream::operator >> (double& v)
{ Read(&v, sizeof(v)); };

////
// WFile object - windows file handling

WFile::WFile()
	: WStream()
{
  m_nCode = 0;
  m_hFile = HFILE_ERROR;
  m_bErr = 0;
}

WFile::WFile(const WChar* lpszFileName, WUInt fuMode, WUInt fuAcc)
	: WStream()
{
  m_nCode = 0;
  m_hFile = HFILE_ERROR;
  m_bErr = 0;
  
  Open(lpszFileName, fuMode, fuAcc);
}

WFile::~WFile()
{
  Close();
}
  
WBool WFile::Open(const WChar* lpszFileName, WUInt fuMode, WUInt fuAcc)
{
	WStream::Open(fuMode & WFile::modeWrite);
	
	if ( !(fuMode & WFile::modeText) )
		fuMode |= WFile::modeBinary;
	
  WASSERT( !((fuMode & WFile::modeRead) && (fuMode && WFile::modeCreate)) );
  
	m_hFile = ::open(lpszFileName, fuMode, fuAcc);
	
	m_bErr = (m_hFile == HFILE_ERROR);
	
	if ( m_bErr )
		SetError(errno);
		
  return !m_bErr;
}

WBool WFile::Create(const WChar* lpszFileName, WUInt fuMode, WUInt fuAcc)
{
	return Open(lpszFileName, fuMode | modeCreate, fuAcc);
}

WBool WFile::Close()
{
	WStream::Close();
	
  if (m_hFile != HFILE_ERROR)
    ::close(m_hFile);
		
  return TRUE;
}

WUInt WFile::Write(void* lpBuf, WUInt nBytes)
{
    WUInt Res;
  WASSERT(m_hFile != HFILE_ERROR);
	
	if (m_hFile == HFILE_ERROR)
		return 0;
	
	if ((Res = ::write(m_hFile, lpBuf, nBytes)) == HFILE_ERROR )
	{
		SetError(errno);
		return 0;
	}	

	m_cbOut += Res;
  return Res;
}

WUInt WFile::Read(void* lpBuf, WUInt nBytes)
{
	WUInt Res;
  WASSERT(m_hFile != HFILE_ERROR);
	
	if (m_hFile == HFILE_ERROR)
		return 0;

  if ((Res = ::read(m_hFile, lpBuf, nBytes)) == HFILE_ERROR )
	{
		SetError(errno);
		return 0;
	}
	
	m_cbIn += Res;
  return Res;
}

WUInt WFile::GetPos()
{
	return ::tell(m_hFile);
}

WLong WFile::Seek(WLong nOffset, SeekOrg nOrg)
{
 	WASSERT(m_hFile != HFILE_ERROR);
  return ( m_hFile == HFILE_ERROR ? 0 : ::lseek(m_hFile, nOffset, nOrg) );
}

WUInt WFile::GetSize()
{
	WASSERT(m_hFile != HFILE_ERROR);
	
	if (m_hFile != HFILE_ERROR)
	{
		WUInt Res, Pos = GetPos();
		if ((Res = ::lseek(m_hFile, 0, WFile::end)) != HFILE_ERROR)
		{
			::lseek(m_hFile, Pos, WFile::beginning);
			return Res;
		}
	}
	
	SetError(errno);	
	return 0;
}

WBool WFile::SetSize(WUInt cbNewSize)
{
	if (m_hFile != HFILE_ERROR)
	{
		int res = ::chsize(m_hFile, (long) cbNewSize);
		
		if (res == 0)
			return TRUE;
	}
	SetError(errno);
	return FALSE;
}

WBool WFile::Eof()
{
	WASSERT(IsOpen());
	
	if (IsOpen())
	{
		int err = ::eof(m_hFile);
		if (err == HFILE_ERROR)
			SetError(err);
		else
			if (err != 0)
				return TRUE;
	}
	return FALSE;
}

void WFile::SetError(int nErr)
{
	WStream::SetError(nErr);
	m_bErr = TRUE;
}

void WFile::Reset()
{
	WStream::Reset();
	m_bErr = FALSE;
}
