//////////////////////////////////////////////////////
/////
////	Optima++ class; tested with Optima++ 1.5
///
//	You may use this source code in any way you like;
//	it has been tested, but may still contain errors.
// 	In any case, its up to the user to make shure this
//	source works correctly for his/her purpose.
// 	In case you have questions concerning the file i/o used, 
//	look at the documentation	for the clib, or email me
//	at mlenzing@aol.com. If you find errors, please mail me
//	so I can correct them in my files and repost the updated files.
//	

#ifndef __OPP_WSTREAMHPP
#define __OPP_WSTREAMHPP

#include <errno.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <fcntl.h>
#include "wflagger.hpp"

////
// stream class

class WStream : public WObject, public WFlagger
{
public:
	enum StreamState {
		stateEnd = 0x001,
		stateError = 0x002,
		stateGood = 0x004,
		stateLoading = 0x0100,
		stateSaving = 0x0200,
		stateOpen = 0x0400
	};

  enum SeekOrg {
    beginning = 0,
    current = 1,
    end = 2
    };

	WStream();
	virtual ~WStream();

	////
	// stream access functions

	virtual WBool Open(WBool bSave);
	virtual WBool Close();

	////
	// stream state functions
	virtual WBool IsGood() { return GetFlag(stateGood); };
	virtual WBool IsAtEnd() { return GetFlag(stateEnd); };
	virtual WBool IsLoading() { return GetFlag(stateLoading); };
	virtual WBool IsSaving() { return GetFlag(stateSaving); };
	
	virtual WBool IsOpen() { return GetFlag(stateOpen); };
	virtual WBool IsClosed() { return !GetFlag(stateOpen); };
	
	virtual void SetError(int nErr);
	virtual int GetError();
	virtual void Reset();
	
  virtual WUInt Write(void* lpBuf, WUInt nBytes)
		{ m_cbOut += nBytes; return 0; };
  virtual WUInt Read(void* lpBuf, WUInt nBytes)
		{ m_cbIn += nBytes; return 0; };

	////
	// shift in/shift out operators:
	
  void operator << (char& v);
  void operator << (int& v);
	void operator << (short& v);
  void operator << (long& v);
	void operator << (unsigned char& v);
	void operator << (unsigned int& v);
  void operator << (unsigned short&  v);
  void operator << (unsigned long& v);
  void operator << (float&  v);
  void operator << (double& v);
	
  
  void operator >> (char& v);
  void operator >> (int& v);
	void operator >> (short& v);
  void operator >> (long& v);
	void operator >> (unsigned char& v);
  void operator >> (unsigned int& v);
	void operator >> (unsigned short& v);
  void operator >> (unsigned long& v);
  void operator >> (float&  v);
  void operator >> (double& v);

	////
	// members:
protected:
	WUInt m_cbIn, m_cbOut;
	int m_nErr;
};

  
////
// file class

#define HFILE					int
#define HFILE_ERROR		-1


class WFile : public WStream
{
public:
  enum OpenFlags {
    modeRead =          O_RDONLY,
    modeWrite =         O_WRONLY,
    modeReadWrite =     O_RDWR,
    modeAppend =       	O_APPEND,
    modeCreate =    		O_CREAT,
    modeTrunc = 			  O_TRUNC,
    modeBinary =   		  O_BINARY,
    modeText = 			    O_TEXT,
		modeNoInherit = 		O_NOINHERIT,
    modeExclusive =     O_EXCL
    };
		
	enum AccessFlags {
		accUserRead	 = 				S_IRUSR,
		accUserWrite = 				S_IWUSR,
		accUserExecSearch = 	S_IXUSR,
		accGroupRead	 = 			S_IRGRP,
		accGroupWrite = 			S_IWGRP,
		accGroupExecSearch = 	S_IXGRP,
		accDefault = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP
		};
		
  enum ErrorFlags {
		errAccess = 			EACCES,		
		errTooManyFiles	=	EMFILE,
		errFileNotFound	=	ENOENT
		};
		
	enum SeekOrg {
		beginning			=	SEEK_SET,
		current				= SEEK_CUR,
		end						= SEEK_END
		};
		
public:
  WFile();
  WFile(const WChar* lpszFileName, WUInt fuMode, WUInt fuAcc = accDefault);
  virtual ~WFile();

public:
	// the file will be opened in binary mode,
	// unless you specify WFile::modeText!
	// there's usually no need to specify the access flags.
  virtual WBool Open(const WChar* lpszFileName, WUInt fuMode, WUInt fuAccess = accDefault);
	virtual WBool Create(const WChar* lpszFileName, WUInt fuMode, WUInt fuAccess = accDefault);
  virtual WBool Close();
  
	////
	// overloads:
	
	virtual WBool IsAtEnd() { return Eof(); };
	virtual void SetError(int nErr);
	virtual void Reset();
	
  virtual WUInt Write(void* lpBuf, WUInt nBytes);
  virtual WUInt Read(void* lpBuf, WUInt nBytes);
  virtual WUInt GetPos();
	virtual WUInt GetSize();
	virtual WBool SetSize(WUInt cbNewSize);
	virtual WBool Eof();
  virtual WLong Seek(WLong nOffset, SeekOrg nOrg = WFile::beginning);

  HFILE GetFileHandle() { return m_hFile; };
  WBool Ok() { return !m_bErr; };
  WUInt GetCode() { return m_nCode; };
	
protected:
  HFILE m_hFile;
  WUInt m_nCode;
  WBool m_bErr;
};


#endif
